@@ -1,20 +1,26 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
+* Copyright (c) Johan Mabille, Sylvain Corlay and Wolf Vollprecht          *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#ifndef XLAYOUT_HPP
-#define XLAYOUT_HPP
+#ifndef XTENSOR_LAYOUT_HPP
+#define XTENSOR_LAYOUT_HPP
+
+// Do not include anything else here.
+// xlayout.hpp is included in xtensor_forward.hpp
+// and we don't want to bring other headers to it.
+#include "xtensor_config.hpp"
 
 namespace xt
 {
     /*! layout_type enum for xcontainer based xexpressions */
     enum class layout_type
     {
-        /*! dynamic layout_type: you can reshape to row major, column major, or use custom strides */
+        /*! dynamic layout_type: you can resize to row major, column major, or use custom strides */
         dynamic = 0x00,
         /*! layout_type compatible with all others */
         any = 0xFF,
@@ -37,7 +43,7 @@ namespace xt
        d = dynamic, a = any, r = row_major, c = column_major.
        @endverbatim
      * Using bitmasks to avoid nested if-else statements.
-     * 
+     *
      * @param args the input layouts.
      * @return the output layout, computed with the previous logical table.
      */
@@ -46,6 +52,8 @@ namespace xt
 
     constexpr layout_type default_assignable_layout(layout_type l) noexcept;
 
+    constexpr layout_type layout_remove_any(const layout_type layout) noexcept;
+
     /******************
      * Implementation *
      ******************/
@@ -84,9 +92,13 @@ namespace xt
     constexpr layout_type default_assignable_layout(layout_type l) noexcept
     {
         return (l == layout_type::row_major || l == layout_type::column_major) ?
-            l : layout_type::row_major;
+            l : XTENSOR_DEFAULT_LAYOUT;
     }
 
+    constexpr layout_type layout_remove_any(const layout_type layout) noexcept
+    {
+        return layout == layout_type::any ? XTENSOR_DEFAULT_LAYOUT : layout;
+    }
 }
 
 #endif
