@@ -1,17 +1,19 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
+* Copyright (c) Johan Mabille, Sylvain Corlay and Wolf Vollprecht          *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#ifndef XVECTORIZE_HPP
-#define XVECTORIZE_HPP
+#ifndef XTENSOR_VECTORIZE_HPP
+#define XTENSOR_VECTORIZE_HPP
 
 #include <type_traits>
 #include <utility>
 
+#include "xfunction.hpp"
 #include "xutils.hpp"
 
 namespace xt
@@ -27,7 +29,7 @@ namespace xt
     public:
 
         template <class... E>
-        using xfunction_type = xfunction<F, R, xclosure_t<E>...>;
+        using xfunction_type = xfunction<F, xclosure_t<E>...>;
 
         template <class Func, class = std::enable_if_t<!std::is_same<std::decay_t<Func>, xvectorizer>::value>>
         xvectorizer(Func&& f);
@@ -52,8 +54,13 @@ namespace xt
     template <class F, class R, class... Args>
     xvectorizer<F, R> vectorize(F&& f, R (*)(Args...));
 
+// Workaround for Visual Studio 15.7.1.
+// Error C2668 (ambiguous call to overloaded function) mistaking a declarations
+// for the definition of another overload.
+#ifndef _MSC_VER
     template <class F>
-    auto vectorize(F&& f) -> decltype(vectorize(std::forward<F>(f), (detail::get_function_type<F>*)nullptr));
+    auto vectorize(F&& f) -> decltype(vectorize(std::forward<F>(f), std::declval<detail::get_function_type<F>*>()));
+#endif
 
     /******************************
      * xvectorizer implementation *
@@ -86,9 +93,9 @@ namespace xt
     }
 
     template <class F>
-    inline auto vectorize(F&& f) -> decltype(vectorize(std::forward<F>(f), (detail::get_function_type<F>*)nullptr))
+    inline auto vectorize(F&& f) -> decltype(vectorize(std::forward<F>(f), std::declval<detail::get_function_type<F>*>()))
     {
-        return vectorize(std::forward<F>(f), (detail::get_function_type<F>*)nullptr);
+        return vectorize(std::forward<F>(f), static_cast<detail::get_function_type<F>*>(nullptr));
     }
 }
 
