@@ -246,7 +246,10 @@ namespace xt
         xt::xarray<double> c = linspace<double>(0., 2., 1, true);
         EXPECT_EQ(c.dimension(), size_t(1));
         EXPECT_EQ(c.shape(), expected_shape);
-        EXPECT_EQ(0., b(0));
+        EXPECT_EQ(0., c(0));
+
+        auto d = linspace<double>(0., 2., 1, true);
+        EXPECT_EQ(c, d);
     }
 
     TEST(xbuilder, linspace_n_samples_endpoint)
@@ -271,6 +274,13 @@ namespace xt
         ASSERT_EQ(m_assigned(3), at_3);
     }
 
+    TEST(xbuilder, linspace_endpoint)
+    {
+        double at_end = 99.78730976641236;
+        xt::xtensor<double, 1> ls = linspace<double>(0., at_end, 100);
+        ASSERT_EQ(ls(99), at_end);
+    }
+
     TEST(xbuilder, linspace_integer)
     {
         xarray<int> ls = linspace<int>(0, 10, 13);
@@ -279,6 +289,15 @@ namespace xt
         ASSERT_TRUE(all(equal(ls, expected)));
     }
 
+    TEST(xbuilder, linspace_double)
+    {
+        // access_imp will also be tested here
+        double at_end = 99.78730976641236;
+        xt::xarray<double> a = xt::linspace(0., at_end, 100);
+        auto b = xt::linspace(0., at_end, 100);
+        EXPECT_EQ(a, b);
+    }
+
     TEST(xbuilder, logspace)
     {
         auto ls = logspace<double>(2., 3., 4);
@@ -683,11 +702,4 @@ namespace xt
         b = std::is_same<decltype(ed3), xarray<double>>::value;
         EXPECT_TRUE(b);
     }
-
-    TEST(xbuilder, linspace_double)
-    {
-        xt::xarray<double> a = xt::linspace(0., 100.);
-        auto b = xt::linspace(0., 100.);
-        EXPECT_EQ(a, b);
-    }
 }
