@@ -1,5 +1,6 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
+* Copyright (c) Johan Mabille, Sylvain Corlay and Wolf Vollprecht          *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -30,15 +31,22 @@ namespace xt
         EXPECT_EQ(rm1, cm1);
         std::vector<double> cm_data = {1, 4, 7, 2, 5, 8, 3, 6, 9};
         std::vector<double> rm_data = {1, 2, 3, 4, 5, 6, 7, 8, 9};
-        EXPECT_FALSE(cm.data() == rm.data());
-        EXPECT_EQ(cm_data, cm.data());
-        EXPECT_EQ(rm_data, rm.data());
-        EXPECT_EQ(rm_data, dm.data());
-        EXPECT_TRUE(cm.data()[1] == 4);
-        EXPECT_TRUE(rm.data()[1] == 2);
+        EXPECT_FALSE(cm.storage() == rm.storage());
+        EXPECT_EQ(cm_data, cm.storage());
+        EXPECT_EQ(rm_data, rm.storage());
+        if (XTENSOR_DEFAULT_LAYOUT == layout_type::row_major)
+        {
+            EXPECT_EQ(rm_data, dm.storage());
+        }
+        else
+        {
+            EXPECT_EQ(cm_data, dm.storage());
+        }
+        EXPECT_TRUE(cm.storage()[1] == 4);
+        EXPECT_TRUE(rm.storage()[1] == 2);
 
         xarray<double, layout_type::column_major> cm_assigned = rm;
-        EXPECT_EQ(cm.data(), cm_assigned.data());
+        EXPECT_EQ(cm.storage(), cm_assigned.storage());
     }
 
     TEST(xlayout, xtensor_initializer_constructors)
@@ -54,15 +62,22 @@ namespace xt
         EXPECT_EQ(rm1, cm1);
         std::vector<double> cm_data = {1, 4, 7, 2, 5, 8, 3, 6, 9};
         std::vector<double> rm_data = {1, 2, 3, 4, 5, 6, 7, 8, 9};
-        EXPECT_FALSE(cm.data() == rm.data());
-        EXPECT_EQ(cm_data, cm.data());
-        EXPECT_EQ(rm_data, rm.data());
-        EXPECT_EQ(rm_data, dm.data());
-        EXPECT_TRUE(cm.data()[1] == 4);
-        EXPECT_TRUE(rm.data()[1] == 2);
+        EXPECT_FALSE(cm.storage() == rm.storage());
+        EXPECT_EQ(cm_data, cm.storage());
+        EXPECT_EQ(rm_data, rm.storage());
+        if (XTENSOR_DEFAULT_LAYOUT == layout_type::row_major)
+        {
+            EXPECT_EQ(rm_data, dm.storage());
+        }
+        else
+        {
+            EXPECT_EQ(cm_data, dm.storage());
+        }
+        EXPECT_TRUE(cm.storage()[1] == 4);
+        EXPECT_TRUE(rm.storage()[1] == 2);
 
         xtensor<double, 2, layout_type::column_major> cm_assigned = rm;
-        EXPECT_EQ(cm.data(), cm_assigned.data());
+        EXPECT_EQ(cm.storage(), cm_assigned.storage());
     }
 
     TEST(xlayout, xfunctions_mix_layouts)
@@ -82,7 +97,7 @@ namespace xt
         EXPECT_TRUE(e(0, 1));
         EXPECT_TRUE(e(1, 1));
 
-        for (auto it = e.xbegin(); it != e.xend(); it++)
+        for (auto it = e.begin(); it != e.end(); it++)
         {
             EXPECT_TRUE(*it);
         }
@@ -104,8 +119,8 @@ namespace xt
         xarray<double, layout_type::row_major> rmt1 = triu(cm);
         xarray<double, layout_type::row_major> rmt2 = triu(rm);
 
-        EXPECT_EQ(cmt1.data(), cmt2.data());
-        EXPECT_EQ(rmt1.data(), rmt2.data());
+        EXPECT_EQ(cmt1.storage(), cmt2.storage());
+        EXPECT_EQ(rmt1.storage(), rmt2.storage());
         EXPECT_TRUE(all(equal(cmt1, cmt2)));
         EXPECT_TRUE(all(equal(rmt1, rmt2)));
 
@@ -143,16 +158,31 @@ namespace xt
         EXPECT_NE(rrm_wrong, rcm);
     }
 
-
     TEST(xlayout, equal_iterator)
     {
         xarray<double, layout_type::column_major> cm = {{1, 2, 3}, {4, 5, 6}, {7, 8, 9}};
         xarray<double, layout_type::row_major> rm = {{1, 2, 3}, {4, 5, 6}, {7, 8, 9}};
         auto e = equal(cm, rm);
-        auto iter_end = e.xend();
-        for (auto iter = e.xbegin(); iter != e.xend(); ++iter)
+        auto iter_end = e.end();
+        for (auto iter = e.begin(); iter != e.end(); ++iter)
         {
             EXPECT_TRUE(*iter);
         }
     }
-}
\ No newline at end of file
+
+    TEST(xlayout, select_layout)
+    {
+        layout_type l = select_layout<layout_type::dynamic, xt::static_shape<std::size_t, 3>>::value;
+        EXPECT_EQ(l, layout_type::dynamic);
+        l = select_layout<layout_type::row_major, xt::static_shape<std::size_t, 3>>::value;
+        EXPECT_EQ(l, layout_type::row_major);
+        l = select_layout<layout_type::row_major, xt::static_shape<std::size_t, 1>>::value;
+        EXPECT_EQ(l, layout_type::any);
+        l = select_layout<layout_type::column_major, xt::static_shape<std::size_t, 1>>::value;
+        EXPECT_EQ(l, layout_type::any);
+        l = select_layout<layout_type::column_major, xt::fixed_shape<>>::value;
+        EXPECT_EQ(l, layout_type::any);
+        l = select_layout<layout_type::column_major, xt::fixed_shape<2, 3>>::value;
+        EXPECT_EQ(l, layout_type::column_major);
+    }
+}
