@@ -22,7 +22,6 @@
 #else
 #endif
 
-
 #include "xtensor/xarray.hpp"
 #include "xtensor/xbuilder.hpp"
 #include "xtensor/xfixed.hpp"
@@ -1034,7 +1033,6 @@ namespace xt
         xtensor<double, 3> exp_v1 = {{{9, 12}, {13, 16}}};
 
         EXPECT_EQ(v1, exp_v1);
-
         test_view_iter(v1, exp_v1);
 
         auto v2 = xt::view(a, keep(1), xt::all(), xt::range(0, xt::xnone(), 3));
@@ -1053,9 +1051,9 @@ namespace xt
         EXPECT_EQ(v4, exp_v4);
 
         v4(0, 0) = 123;
-        v4(1, 0) = 123;
+        v4(1, 0, 0) = 123;
         EXPECT_EQ(a(0, 0, 0), 123);
-        EXPECT_EQ(a(1, 0, 0), 123);
+        EXPECT_EQ(a(2, 0, 0), 123);
 
         v3(0, 2, 1) = 1000;
         EXPECT_EQ(a(1, 1, 3), 1000);
@@ -1113,9 +1111,9 @@ namespace xt
         EXPECT_EQ(v4, exp_v4);
 
         v4(0, 0) = 123;
-        v4(1, 0) = 123;
+        v4(1, 0, 0) = 123;
         EXPECT_EQ(a(0, 0, 0), 123);
-        EXPECT_EQ(a(1, 0, 0), 123);
+        EXPECT_EQ(a(2, 0, 0), 123);
 
         bool b = detail::is_strided_view<decltype(a), xkeep_slice<int>, int>::value;
         EXPECT_FALSE(b);
@@ -1464,6 +1462,35 @@ namespace xt
         EXPECT_EQ(a, b);
     }
 
+    TEST(xview, front)
+    {
+        xt::xtensor<size_t,2> a = {{1,2,3}, {4,5,6}};
+        auto v = xt::view(a, 0, xt::all());
+        auto w = xt::view(a, 1, xt::all());
+        EXPECT_EQ(v.front(), 1);
+        EXPECT_EQ(w.front(), 4);
+    }
+
+    TEST(xview, back)
+    {
+        xt::xtensor<size_t,2> a = {{1,2,3}, {4,5,6}};
+        auto v = xt::view(a, 0, xt::all());
+        auto w = xt::view(a, 1, xt::all());
+        EXPECT_EQ(v.back(), 3);
+        EXPECT_EQ(w.back(), 6);
+    }
+
+    TEST(xview, flat)
+    {
+        xt::xtensor<size_t, 2, xt::layout_type::row_major> a = {{0,1,2}, {3,4,5}};
+        xt::xtensor<size_t, 2, xt::layout_type::row_major> b = {{0,1,2}, {30,40,50}};
+        auto view = xt::view(a, 1, xt::all());
+        view.flat(0) = 30;
+        view.flat(1) = 40;
+        view.flat(2) = 50;
+        EXPECT_EQ(a, b);
+    }
+
     TEST(xview, in_bounds)
     {
         xt::xtensor<size_t,2> a = {{0,1,2}, {3,4,5}};
@@ -1586,8 +1613,6 @@ namespace xt
             { 3, 4 },
         };
 
-        std::cout << tensor.shape().size() << std::endl;
-
         const auto row0 = xt::row(tensor, 0);
         const auto row1 = xt::row(tensor, 1);
 
