@@ -1,5 +1,6 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
+* Copyright (c) Johan Mabille, Sylvain Corlay and Wolf Vollprecht          *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -17,11 +18,11 @@ namespace xt
     template <class F, class C>
     struct view_op_tester : operation_tester<F, C>
     {
-        using container_type = C;
-        container_type vres_rr;
-        container_type vres_rc;
-        container_type vres_rct;
-        container_type vres_ru;
+        using storage_type = C;
+        storage_type vres_rr;
+        storage_type vres_rc;
+        storage_type vres_rct;
+        storage_type vres_ru;
 
         size_t x_slice;
         xrange<size_t> y_slice;
@@ -75,11 +76,11 @@ namespace xt
     {
     public:
 
-        using container_type = C;
+        using storage_type = C;
     };
 
     using testing_types = ::testing::Types<xarray_dynamic, xtensor_dynamic>;
-    TYPED_TEST_CASE(view_semantic, testing_types);
+    TYPED_TEST_SUITE(view_semantic, testing_types);
 
     TYPED_TEST(view_semantic, a_plus_b)
     {
@@ -252,7 +253,6 @@ namespace xt
     TYPED_TEST(view_semantic, a_plus_equal_b)
     {
         view_op_tester<std::plus<>, TypeParam> t;
-        auto viewa = view(t.a, t.x_slice, t.y_slice, t.z_slice);
 
         {
             SCOPED_TRACE("row_major += row_major");
@@ -294,7 +294,6 @@ namespace xt
     TYPED_TEST(view_semantic, a_minus_equal_b)
     {
         view_op_tester<std::minus<>, TypeParam> t;
-        auto viewa = view(t.a, t.x_slice, t.y_slice, t.z_slice);
 
         {
             SCOPED_TRACE("row_major -= row_major");
@@ -336,7 +335,6 @@ namespace xt
     TYPED_TEST(view_semantic, a_times_equal_b)
     {
         view_op_tester<std::multiplies<>, TypeParam> t;
-        auto viewa = view(t.a, t.x_slice, t.y_slice, t.z_slice);
 
         {
             SCOPED_TRACE("row_major *= row_major");
@@ -378,7 +376,6 @@ namespace xt
     TYPED_TEST(view_semantic, a_divide_by_equal_b)
     {
         view_op_tester<std::divides<>, TypeParam> t;
-        auto viewa = view(t.a, t.x_slice, t.y_slice, t.z_slice);
 
         {
             SCOPED_TRACE("row_major /= row_major");
@@ -421,8 +418,8 @@ namespace xt
     {
         using container_1d = redim_container_t<TypeParam, 1>;
         using container_2d = redim_container_t<TypeParam, 2>;
-        container_2d a = {{1, 2, 3, 4},
-                          {5, 6, 7, 8},
+        container_2d a = {{1,  2,  3,  4},
+                          {5,  6,  7,  8},
                           {9, 10, 11, 12}};
         container_2d b = a;
         auto viewa = view(a, all(), range(1, 4));
@@ -453,4 +450,16 @@ namespace xt
 
         EXPECT_EQ(res, a);
     }
+
+    TYPED_TEST(view_semantic, higher_dimension_broadcast)
+    {
+        using container_2d = redim_container_t<TypeParam, 2>;
+
+        container_2d a = { {1, 2, 3}, {4, 5, 6} };
+        container_2d b = { {11, 12, 13} };
+        container_2d res = { { 11, 12, 13 }, { 4, 5, 6 } };
+
+        auto viewa = view(a, 0, all());
+        XT_EXPECT_ANY_THROW(viewa = b);
+    }
 }
